; asmsyntax=pic16f84
;
; File: uct.h
; Compiler: mplab v6.30
;
; MCU: PIC-16F84A @ 4 MHz
; Date: 18/09/2003
; Update: 15/05/2004
; Author: Boris Estudiez
;
; DESCRIPCION:
; Definicion de Constantes/Macros/Variables para uct.asm.

;Definicion de Macros
#define BANK_0 bcf STATUS, RP0 
#define BANK_1 bsf STATUS, RP0 

;******************************************************************
;* Definicion de "variables estaticas" en EEPROM .                *
;******************************************************************

EEPROM_ADDR:   ORG 0x2100
EE_RINGS       equ   0x00 ;Cantidad de RINGS para atender llamado.    
               de    0x04 ;Valor de fabrica: 4 RINGS.
EE_PASSW_0     equ   0x01 ;Digito 3 y 2 de la clave de acceso (MSB)   
               de    0x12 ;Valor de fabrica: 1 y 2  
EE_PASSW_1     equ   0x02 ;Digito 1 y 0 de la clave de acceso (LSB)   
               de    0x34 ;Valor de fabrica: 3 y 4
EE_DISPSTAT    equ   0x03 ;Ultimo Estado de Dispositivos ON/OFF 0000xxxx
               de    0x00 ;Valor de fabrica: Todos disp. desactivados.

;******************************************************************
;* Definicion de constantes del programa de control del UCT       *
;******************************************************************

;Definicion de Nombres de Variables en RAM
ax             equ 0x20 ;Variable de proposito general
bx             equ 0x21 ;Variable de proposito general
cx             equ 0x22 ;Variable de proposito general
dx             equ 0x23 ;Variable de proposito general
ex             equ 0x24 ;Variable de proposito general
UCT_STATUS     equ 0x30 ;Registro de estado del UCT.
                        ;Bit0=RINGING. Intento de llamada. 
                        ;Bit1=ANSWERCALL. Respondiendo llamada.
                        ;Bit2=DCEINT. Interrupcion por DCE.
                        ;Bit3=TIMEOUT. Ocurrio un time-out. 
                        ;(se debe borrar antes luego de ser usada).
UCT_OPTION     equ 0x31 ;Opciones de funcionamiento del UCT .
                        ;Bit3=ISD_SURE_RESET. Reset seguro de puntero 
                        ;     de direcciones para ISD1420.
                        ;Bit4=SKIPMENU. Evita repro. mensages de menu.
                        ;Bit5=QF_DTMF. Salir de funcion al recibirse un
                        ;     codigo DTMF. Solo valido para mpause().
                        ;Bit6=QMSG. Reiniciar reproduciendo mensage/aviso.
                        ;Bit7=QTIMEOUT. Reiniciar al producirse un TIMEOUT.
RINGS_LEFT     equ 0x32 ;Numero de RINGS que faltan para contestar llamado.
TMR0_AUX       equ 0x46 ;Variable auxiliar para ser usada como contador
                        ;del timer.
SECONDS        equ 0x47 ;Almacena numero de segundos que se deben 
                        ;temporizar. Usada por set_timeout.
TEMPX          equ 0x48 ;Variable para almacenar valor temporal.
                        ;No usar. P/uso interno de funciones del stack.
TEMP_ax        equ 0x49 ;Ultimas 7 direcciones reservadas para usar
TEMP_bx        equ 0x4A ;con funciones pop_* y push_*
TEMP_cx        equ 0x4B ;Ambas manejan el stack de profundidad: 1
TEMP_dx        equ 0x4C
TEMP_ex        equ 0x4D
TEMP_W         equ 0x4E
TEMP_STATUS    equ 0x4F

;Definicion de constantes numericas del programa.
DTMF_AST       equ 0x0B  ;Codigo del DTMF Asterisco.
DTMF_NUM       equ 0x0C  ;Codigo del DTMF Numeral.

_LOT_RINGS     equ d'18' ;Permite poner 18 RINGS. Ver conf_ring_menu().

ERR            equ 0xFF  ;Operacion no exitosa. 
OK             equ 0x00  ;Operacion exitosa.

;Ctes. usadas por UCT_OPTION - UCT_STATUS (Refieren a posicion de bit)
;UCT_OPTION bits:
QTIMEOUT       equ 7    
QMSG           equ 6
QF_DTMF        equ 5
SKIPMENU       equ 4
ISD_SURE_RESET equ 3

;UCT_STATUS bits:
RINGING        equ 0
ANSWERCALL     equ 1
DCEINT         equ 2
TIMEOUT        equ 3

;Ctes. de tiempo, pueden usarse con set_timeout().
M04            equ d'240'  ;240 segundos -> 4 Minutos.
S60            equ d'60'   ; 60 segundos -> 1 Minuto.
S30            equ d'30'   ; 30 segundos 
S10            equ d'10'   ; 10 segundos
S2             equ d'2'    ;  2 segundos
S1             equ d'1'    ;  1 segundos

;Ctes. de tiempo, usadas por play_isd().
ISD_SURE_RTIME equ d'3'    ; ISD_RESET_RTIME*0xFF milisegundos.

;Ctes. usadas por el BUS DCE (DCE = Dispositivo de Control Externo)

;Pines del BUS DCE.
;-->Interrupciones (utilizan logica negada):
#define        DCEINT_PIN    PORTB, 2  ;Interrupme a un DCE.
#define        UCTINT_PIN    PORTB, 1  ;Comprueba Interrupcion por DCE.   


;******************************************************************
;* Numeros de posicion Asociados a los Mensages Almacenados en la * 
;* memoria ISD1420.                                               *
;******************************************************************

;Silencio (primeros dos mensages), Duracion 25 ms cada uno.
ISD_SILENCIO_1             equ d'1'
ISD_SILENCIO_2             equ d'2'

;Generales/Genericos
ISD_NUMERO                 equ d'3'
ISD_INCORRECTO             equ d'4'
ISD_INGRESAR               equ d'5'
ISD_DISPOSITIVO            equ d'6'
ISD_LISTO                  equ d'7'
ISD_REPETIR                equ d'8'
ISD_DESACTIVADO            equ d'9'
ISD_ACTIVADO               equ d'10'
ISD_CLAVE                  equ d'11'
ISD_CAMBIAR                equ d'12'
ISD_SALIR                  equ d'13'
ISD_CONFIGURAR             equ d'14'
ISD_ERROR                  equ d'15' 

;Menus
ISD_CONTROL_DE             equ d'16'
ISD_VER_ESTADO_DE          equ d'17'
ISD_BUS_DCE                equ d'18'

;Teclas
ISD_UNO                    equ d'19'
ISD_DOS                    equ d'20'
ISD_TRES                   equ d'21'
ISD_CUATRO                 equ d'22'
ISD_CINCO                  equ d'23'
ISD_ASTERISCO              equ d'24'
ISD_NUMERAL                equ d'25'

;Especiales
ISD_EXTERNO                equ d'26'
ISD_CONECTAR               equ d'27'
ISD_RINGS                  equ d'28'
ISD_TELEFONICO             equ d'29'
ISD_ALARMA_2               equ d'30'

;Otros
ISD_BIENVENIDO             equ d'31'

