; asmsyntax=pic16f84
;
; File    : uct.inc
; Compiler: mplab v6.61
;
; MCU     : PIC-16F84A @ 4 MHz
; Created : 18/Sep/2003
; Revision: 05/Jan/2005
;
; Author  : Boris Estudiez <stk@freeshell.org>
; Licence : GPL
;
; DESCRIPCION:
; Definicion de Constantes/Macros/Variables para uct.asm.
;
; (c) Copyright slicetex electronixs (2001-2005)
;     http://stk.freeshell.org
;


;******************************************************************
;* Definicion de Macros Generales                                 *
;******************************************************************
#define BANK_0 bcf STATUS, RP0 
#define BANK_1 bsf STATUS, RP0 

;******************************************************************
;* Definicion de "variables estaticas" en EEPROM .                *
;*                                                                *
;* PRIMEROS 15 BYTES RESERVADOS PARA CONF. DEL UCT                *
;******************************************************************

;[0x00 - 0x02] ID DEL DISPOSITIVO: 
;ID_2:ID_1:ID_0 = 0x000258 = 600 -> STX600

EEPROM_ADDR:   ORG 0x2100
STX_ID_0       equ   0x00 ;ID_0 del dispositivo
               de    0x58 ;ID_0 = 0x58
STX_ID_1       equ   0x01 ;ID_1 del dispositivo
               de    0x02 ;ID_1 = 0x02
STX_ID_2       equ   0x02 ;ID_2 del dispositivo
               de    0x00 ;ID_2 = 0x00

;[0x03 - 0x05] VERSION DEL FIRMWARE:
;FV_2:FV_1:FV_0 = '0' '8' '1' -> 0.8.1,  en ASCII.

FV_2           equ   0x03 ;FV_0 del dispositivo
               de    '0'  ;FV_0 = '0'
FV_1           equ   0x04 ;FV_1 del dispositivo
               de    '8'  ;FV_1 = '8'
FV_0           equ   0x05 ;FV_2 del dispositivo
               de    '1'  ;FV_2 = '1'

;[0x06 - 0x0E] CONFIGURACION DEL UCT:

EE_RINGS       equ   0x06 ;Cantidad de RINGS para atender llamado.    
               de    d'4' ;Valor de fabrica: 4 RINGS.
EE_PASSW_0     equ   0x07 ;Digito 3 y 2 de la clave de acceso (MSB)   
               de    0x12 ;Valor de fabrica: 1 y 2  
EE_PASSW_1     equ   0x08 ;Digito 1 y 0 de la clave de acceso (LSB)   
               de    0x34 ;Valor de fabrica: 3 y 4
EE_DISPSTAT    equ   0x09 ;Ultimo Estado de Dispositivos ON/OFF 0000xxxx
               de    0x00 ;Valor de fabrica: Todos disp. desactivados.

;[0xF - 0x3F] LIBRES.

;******************************************************************
;* Declaracion de Nombres de Variables en RAM                     *
;******************************************************************
ax             equ 0x20 ;Variable de proposito general
bx             equ 0x21 ;Variable de proposito general
cx             equ 0x22 ;Variable de proposito general
dx             equ 0x23 ;Variable de proposito general
ex             equ 0x24 ;Variable de proposito general
UCT_STATUS     equ 0x30 ;Registro de estado del UCT.
                        ;Bit0=RINGING. Intento de llamada. 
                        ;Bit1=ANSWERCALL. Respondiendo llamada.
                        ;Bit2=DCEINT. Interrupcion por DCE.
                        ;Bit3=TIMEOUT. Ocurrio un time-out. 
                        ;(se debe borrar antes luego de ser usada).
UCT_OPTION     equ 0x31 ;Opciones de funcionamiento del UCT .
                        ;Bit3=ISD_SURE_RESET. Reset seguro de puntero 
                        ;     de direcciones para ISD1420.
                        ;Bit4=SKIPMENU. Evita repro. mensages de menu.
                        ;Bit5=QF_DTMF. Salir de funcion al recibirse un
                        ;     codigo DTMF. Solo valido para mpause().
                        ;Bit6=QMSG. Reiniciar reproduciendo mensage/aviso.
                        ;Bit7=QTIMEOUT. Reiniciar al producirse un TIMEOUT.
RINGS_LEFT     equ 0x32 ;Numero de RINGS que faltan para contestar llamado.

                        ;DIRECCIONES 0x33 a 0x39 reservadas para el
                        ;Bus DCE. No USAR!. Ver mas abajo.

TMR0_AUX       equ 0x46 ;Variable auxiliar para ser usada como contador
                        ;del timer.
SECONDS        equ 0x47 ;Almacena numero de segundos que se deben 
                        ;temporizar. Usada por set_timeout.
TEMPX          equ 0x48 ;Variable para almacenar valor temporal.
                        ;No usar. P/uso interno de funciones del stack.
TEMP_ax        equ 0x49 ;Ultimas 7 direcciones reservadas para usar
TEMP_bx        equ 0x4A ;con funciones pop_* y push_*
TEMP_cx        equ 0x4B ;Ambas manejan el stack de profundidad: 1
TEMP_dx        equ 0x4C
TEMP_ex        equ 0x4D
TEMP_W         equ 0x4E
TEMP_STATUS    equ 0x4F

;******************************************************************
;* Definicion de constantes numericas del programa.               *
;******************************************************************

DTMF_AST       equ 0x0B  ;Codigo del DTMF Asterisco.
DTMF_NUM       equ 0x0C  ;Codigo del DTMF Numeral.

_LOT_RINGS     equ d'18' ;Permite poner 18 RINGS. Ver conf_ring_menu().

ERR            equ 0xFF  ;Operacion no exitosa. 
OK             equ 0x00  ;Operacion exitosa.

;UCT_OPTION BITS:
QTIMEOUT       equ 7    
QMSG           equ 6
QF_DTMF        equ 5
SKIPMENU       equ 4
ISD_SURE_RESET equ 3

;UCT_STATUS BITS:
TIMEOUT        equ 3
DCEINT         equ 2
ANSWERCALL     equ 1
RINGING        equ 0

;CTES. DE TIEMPO - Pueden usarse con set_timeout().
M04            equ d'240'  ;240 segundos -> 4 Minutos.
S60            equ d'60'   ; 60 segundos -> 1 Minuto.
S30            equ d'30'   ; 30 segundos 
S10            equ d'10'   ; 10 segundos
S2             equ d'2'    ;  2 segundos
S1             equ d'1'    ;  1 segundos

;CTES. DE TIEMPO - Usadas por play_isd().
ISD_SURE_RTIME equ d'3'    ; ISD_RESET_RTIME*0xFF milisegundos.

;******************************************************************
;* DEFINICIONES PARA EL BUS DCE                                   *
;* DCE = Dispositivo de Control Externo                           * 
;* (Variables, Constantes, Macros)                                *
;******************************************************************

;PINES DEL BUS DCE.
;Interrupciones (utilizan logica negada):
#define        DCEINT_PIN    PORTB, 2  ;Interrupme a un DCE.
#define        UCTINT_PIN    PORTB, 1  ;Comprueba Interrupcion por DCE.   
#define        TXPIN         PORTB, 4  ;Pto. para transmision serie (TX).
#define        RXPIN         PORTB, 5  ;Pto. para recepcion serie (RX).  

;******************************************************************
;* TRANSMISION SERIE ASINCORNA (1200 BAUD, 8N1).                  *
;******************************************************************

;VARIABLES PARA LA TRANSMISION SERIE, ASINCRONA:  
RXTX_BUF       equ 0x33                ;RX/TX BUFFER.
BCOUNTER       equ 0x34                ;BIT COUNTER.
SDCNT          equ 0x35                ;SERIAL DELAY COUNTER.

;CTES. PARA LA TRANSMISION SERIE, ASINCRONA:
BITS           equ d'8'                ;Numbers of bits to transmit.


;MACROS PARA LA TRANSMISION SERIE, ASINCRONA:

;delay_HalfBit(): produce un retardo de 416 uS o de Medio Bit.
delay_HalfBit macro
   call HalfBit_Pause
   endm
;delay_FullBit(): produce un retardo de 832 uS o de Un Bit.
delay_FullBit macro
   call HalfBit_Pause
   call HalfBit_Pause
   endm

;******************************************************************
;* CTP (COMMAND TRANSMISSION PROTOCOL).                           *
;******************************************************************

;VARIABLES: usadas por funciones del CTP.  
B0             equ 0x36  ;BYTE 0 De Trama.
B1             equ 0x37  ;BYTE 1 De Trama.
B2             equ 0x38  ;BYTE 2 De Trama.
CTPSTAT        equ 0x39  ;Registro de estado del CTP.

;CTPSTAT BITS:
CTP_ERR        equ 0     ;1 -> Error En Conexion.
RESET_RCV      equ 1     ;1 -> Comando UCT_RESET Recibido.
END_RCV        equ 2     ;1 -> Comando END_CTP Recibido.
GM_MENU_RCV    equ 3     ;1 -> Comando GOTO_MAIN_MENU Recibido.
CMD_NOT_FOUND  equ 7     ;1 -> Comando No existente.

;CONSTANTES: usadas por funciones del CTP.  
RXWAIT         equ d'30' ;Espera en segundos para la recepcion de 1 Byte.
TRDELAY        equ d'3'  ;Retardo en mS para la transmision de una trama.
WAITWAKEUP     equ d'100';100 mS de espera para que el DCE o UCT despierte.

HELOLSB        equ 0xAA  ;Byte 0 de trama HELO
HELOMSB        equ 0x55  ;Byte 1 de trama HELO

;******************************************************************
;* COMANDOS soportados por el UCT para ser ejecutados a traves    *
;* del BUS DCE.                                                   *
;* Utilizados en el byte CMD de la trama COMMAND del protocolo    *
;* CTP.                                                           *
;* Ver funcion uct_ctpd() en uct.asm, para ver descripcion        *
;* detallada de los comandos.                                     *
;******************************************************************

END_CTP         equ 0xF5  ;El UCT termina la comunicacion con DCE.
UCT_RESET       equ 0x99  ;El UCT se resetea y vuelve a modo espera.

FREE_PHONE_LINE equ 0x98  ;El UCT desocupa linea telefonica.   
GET_PHONE_LINE  equ 0x88  ;El UCT ocupa linea telefonica. 
DIALP_NUMBER    equ 0xCC  ;El UCT marca el numero B1 por Pulsos.
PING            equ 0x87  ;No hace nada, solo devuelve PING. En B0.
UCT_PAUSE       equ 0x55  ;El UCT se queda pausado por B1 segundos.
PLAYISD         equ 0x66  ;Repoduce el mensage B2 de la ISD1420. 
WR_EEPROM       equ 0x77  ;Escribir B1 en direccion B2 de eeprom.
RD_EEPROM       equ 0x44  ;Leer direccion B1 de eeprom.
WR_RAM          equ 0x3A  ;Escribir B1 en direccion B2 de la RAM.
RD_RAM          equ 0x13  ;Leer direccion B1 de la RAM.
GETDTMF         equ 0xEE  ;Esperar por B1 segundos un codigo DTMF.
GETDTMFSTR      equ 0xDD  ;Esperar 1 minuto por B1 codigos/Digitos DTMF.
CTRLDISP        equ 0xBB  ;Ver ctrl_disp(). Argumento en B1.
GOTO_MAIN_MENU  equ 0x93  ;El UCT toma el control y repoduce el menu principal.


;******************************************************************
;* Numeros de posicion Asociados a los Mensages Almacenados en la * 
;* memoria ISD1420.                                               *
;******************************************************************

;Silencio (primeros dos mensages), Duracion 25 ms cada uno.
ISD_SILENCIO_1             equ d'1'
ISD_SILENCIO_2             equ d'2'

;Generales/Genericos
ISD_NUMERO                 equ d'3'
ISD_INCORRECTO             equ d'4'
ISD_INGRESAR               equ d'5'
ISD_DISPOSITIVO            equ d'6'
ISD_LISTO                  equ d'7'
ISD_REPETIR                equ d'8'
ISD_DESACTIVADO            equ d'9'
ISD_ACTIVADO               equ d'10'
ISD_CLAVE                  equ d'11'
ISD_CAMBIAR                equ d'12'
ISD_SALIR                  equ d'13'
ISD_CONFIGURAR             equ d'14'
ISD_ERROR                  equ d'15' 

;Menus
ISD_CONTROL_DE             equ d'16'
ISD_VER_ESTADO_DE          equ d'17'
ISD_BUS_DCE                equ d'18'

;Teclas
ISD_UNO                    equ d'19'
ISD_DOS                    equ d'20'
ISD_TRES                   equ d'21'
ISD_CUATRO                 equ d'22'
ISD_CINCO                  equ d'23'
ISD_ASTERISCO              equ d'24'
ISD_NUMERAL                equ d'25'

;Especiales
ISD_EXTERNO                equ d'26'
ISD_CONECTAR               equ d'27'
ISD_RINGS                  equ d'28'
ISD_TELEFONICO             equ d'29'
ISD_ALARMA_2               equ d'30'

;Otros
ISD_BIENVENIDO             equ d'31'

