; asmsyntax=pic16f84
;
; File     : uct.asm
; MCU      : PIC-16F84A @ 4 MHz
; Date     : 18/09/2003
; Update   : 15/05/2004
; Version  : 0.6.0
; Author   : Boris Estudiez.
; Licence  : GPL
;
; DESCRIPTION:
; Software de control del UCT (Unidad Controlada por Telefono).
;
; HARDWARE VERSION: UCT HV-0.5 o superior.
;
; CONFIGURATION BITS: WDT=ON, OSC=XT, PWRTE=ON, CP=OFF.
; EEPROM DATA VALUES: Mire el archivo README.TXT que se adjunta.  
; 
; (c) Copyright slicetex electronixs (2001-2004)
;     http://stk.freeshell.org
;

   LIST P=PIC16F84A
   #include <p16f84a.inc>
   #include "uct.inc"

   __CONFIG _PWRTE_ON & _WDT_ON & _XT_OSC & _CP_OFF

;*********************************************************************            
;*                       INICIO DEL CODIGO DE PROGRAMA               *
;*********************************************************************            
RESET: ORG 0x00
   goto SETUP           

;*********************************************************************            
;*                       RUTINAS DE ATENCION A INTERRUPCIONES        *
;*********************************************************************            

;*********************************************************************            
;* RSI: Rutina de Servicio a Interrupciones.                         *
;*********************************************************************            
RSI: ORG 0x04   

   bcf INTCON, GIE       ;Inhibir toda interrupcion
   call push_STATUS      ;Salvamos registro STATUS y W.
   call push_w    

   btfsc INTCON, INTF    ;Int. externa?
      goto int_ext       ;Si, determinar tipo de int. externa.
   btfsc INTCON, T0IF    ;Int. TMR0 Overflow?
      goto RSITMR0       ;Si, atender int. por TMR0.
   goto end_RSI          ;No, devolver control a programa.

int_ext:
   btfss UCTINT_PIN      ;Interrupcion por BUS DCE ? 
      goto RSI_DCE       ;Si, int. por BUS DCE. -> UCTINT_PIN=0.
   goto RSILL            ;No, int. por Detector de llamada. Atender.

end_RSI:
   call pop_w            ;Reponer W y STATUS.
   call pop_STATUS
   retfie                ;Devolver control.

;*********************************************************************            
;* RSILL: Rutina de Servicio a Interrupcion por LLamada              *
;* Modifica: RINGS_LEFT, TMR0 y UCT_STATUS.                          *
;*********************************************************************            
RSILL:
   bcf INTCON, INTF           ;INTF=0, reponer flag.

   bsf UCT_STATUS, RINGING    ;Intento de llamada. RINGING=1
   movlw S10
   call set_timeout           ;Seteo timeout en 10 segundos.
   decf RINGS_LEFT, F         ;RINGS_LEFT--

end_RSILL:
   goto end_RSI               ;Devolver control a programa.

;*********************************************************************               
;* RSITMR0: Rutina de Servicio a Interrupcion por TMR0 Overflow.     *   
;* Dependiendo de los valores de los bits del UCT_OPTION toma        *
;* una accion. Este servicio solo puede ser usado con                *
;* la funcion set_timeout().                                         *
;* SI:                                                               *
;* QTIMEOUT-> 1: vuelve a stand-by ; 0:vuelve al programa.           *
;*                                                                   *   
;* Setea con 1 el flag TIMEOUT de UCT_STATUS al ocurrir un timeout.  *
;* Nota: esta rutina deja seteado el Banco 0.                        *
;*********************************************************************            
RSITMR0:
   bcf INTCON, T0IF           ;T0IF=0, Reponer flag.

   BANK_0
   movlw 0x06                 ;Recordar que TMR0 esta en el banco 0.
   movwf TMR0                 ;Proxima interrupcion en 250 cuentas.

   decfsz TMR0_AUX, F         ;TMR0_AUX == 0?, paso 1 segundo?
      goto end_RSITMR0        ;No, seguir esperando.
   movlw d'125'               ;Si cargar TMR0_AUX con 125 y decrementar
   movwf TMR0_AUX             ;SECONDS.
   decfsz SECONDS, F          ;SECONDS==0? TIMEOUT?
      goto end_RSITMR0        ;No, seguir esperando.
   bsf UCT_STATUS, TIMEOUT    ;Si, setear TIMEOUT.
   btfsc UCT_OPTION, QTIMEOUT ;QTIMEOUT==1?
      call close_call         ;Si, inicializar UCT -> volver a stand-by.

end_RSITMR0:
   goto end_RSI               ;devolver control.

;*********************************************************************            
;* RSI_DCE: Rutina de Servicio a Interrupcion por bus DCE.           *
;* NOTA: Esta funcionalidad no esta habilitada aun en el UCT.        *
;* No poner el pin UCTINT a cero en el bus DCE hasta que             *
;* sean completadas las rutinas del BUS DCE.                         *
;*********************************************************************            
RSI_DCE:
   bcf INTCON, INTF              ;INTF=0, reponer flag.
   bsf UCT_STATUS, DCEINT        ;Int. por bus DCE. DCEINT=1.
   
end_RSI_DCE:
   goto end_RSI                  ;Devolver control a programa.

;*********************************************************************            
;*                  RUTINA PRINCIPALES DEL UCT                       *
;*********************************************************************            
;* Notas de programacion:                                            *
;* 1) Banco 0 se supone siempre por defecto, si se cambia a          *
;*    banco 1 en una rutina, luego al salir, volver a poner a 1.     *
;* 2) QTIMEOUT se supone siempre 1, si se pone a 0 en una rutina     *
;*    luego al salir de rutina volver a poner a 1.                   *
;*********************************************************************
            
;*********************************************************************            
;* SETUP: Seteamos Condiciones Iniciales de Funcionamiento.          *
;*********************************************************************            
SETUP:       
   
   clrf INTCON       ;Impedimos toda interrupcion inicialmente.

   BANK_1            ;accedemos al banco 1 de la mem. ram 
   ;Config. Puerto A
   movlw b'00010000' ;A7-5=X,A4=E,A3=S,A2=S,A1=S,A0=S
   movwf TRISA
   ;Config. Puerto B
   movlw b'00000011' ;B7-4=S,B3=S,B2=S,B1=E,B0=E
   movwf TRISB

   ;Conf. OPTION_REG Reg.
   movlw b'1000111'
   movwf OPTION_REG  ;RBPU=OFF PSA=ON->TMR0 DIV=111, INTEDG=0 FALL EDGE.

   BANK_0            ;accedemos al banco 0 de la mem. ram 
   ;Valor inicial Puerto A
   ;A0 | A1 | A2 | A3 | Poner Dispositivo en el bus RB4-7:
   ; 1 |  0 |  0 |  0 |8870 (DTMF DECODER)
   ; 0 |  1 |  0 |  0 |ISD1420/4016 (Memoria De Voz)
   ; 0 |  0 |  1 |  0 |4042 (LATCH/DISP. ON OFF)
   ; 0 |  0 |  0 |  1 |BUS DCE/4016
   ; 0 |  0 |  0 |  0 |Ningun Dispositivo.
   clrf PORTA        ;Limpiamos Latch's de puerto A, Salidas=0. 

   ;Valor inicial Puerto B
   ;Telefono/Linea Desocupado.
   clrf PORTB        ;Limpiamos Latch's de puerto B, Salidas=0
   bsf DCEINT_PIN    ;DCEINT_PIN=1 -> RB2=1.

   ;Actualizamos estado de Dispositivos ON/OFF.
   movlw 0x05        ;W > 4.
   call ctrl_disp    ;Actualizar Latch 4042. Ver ctrl_disp().

   ;Borramos registro de estado y opciones del UCT
   clrf UCT_STATUS
   clrf UCT_OPTION           ;SKIPMENU=0, QMSG=0, QF_DTMF=0.
   bsf  UCT_OPTION, QTIMEOUT ;QTIMEOUT=1 -> salir si hay timeout.
                             ;Por lo general siempre esta activado.
   ;Seteamos variables
   movlw EE_RINGS    
   call eeprom_read     ;W=*EE_RINGS
   movwf RINGS_LEFT     ;Numero de RINGS que faltan para contestar llamado.

   ;Activar Interrupcion solo externa por RB0/INT
   movlw b'10010000' 
   movwf INTCON      ;GIE=1, INTE=1

;*********************************************************************            
;* standby_code: Mantiene al UCT en standby a la espera de algun     *
;* evento externo.                                                   *
;*********************************************************************            
standby_code:
   sleep                     ;Permanecer en Stand-by
   clrwdt                    ;Limpiamos el Watchdog Timer

   btfsc UCT_STATUS, RINGING ;Intento de llamada? RINGING==1
      goto  wait_rings       ;Si, esperar RINGS.
   btfsc UCT_STATUS, DCEINT  ;Interrupcion por DCE? DCEINT==1
      goto dce_connection    ;Si, establecer coneccion con DCE
   goto  standby_code        ;No, permanecer en stand-by.

wait_rings:
      ;Esperamos a que RINGS_LEFT sea cero, antes de atender llamado.
      clrwdt
      movf RINGS_LEFT, F
      btfss STATUS, Z      ;RINGS_LEFT == 0?
         goto wait_rings   ;No, segir esperando.
      call unset_timeout   ;Si, sacamos TIMEOUT introducido por RSILL
      goto reply_call      ;y Respondemos Llamada.


;*********************************************************************            
;*                 RUTINAS DE ATENCION DE LLAMADO DEL UCT            *
;*********************************************************************            
  
;*********************************************************************            
;* reply_call: establece la comunicacion entre el UCT y el usuario   *
;* que llama al UCT.                                                 *
;*********************************************************************            
reply_call:
   
   BANK_0  
   bcf INTCON, INTE            ;Inhibir int. externas, bus DCE inhibido.
   bcf UCT_STATUS, RINGING     ;RINGING=0
   bsf UCT_STATUS, ANSWERCALL  ;ANSWERCALL=1 - UCT respondiendo llamada.
   bsf UCT_OPTION, QMSG        ;QMSG=1, Desconectar con mensage de aviso.

   bsf PORTB, 3      ;RB3=1, Ocupar linea, al circ. selector de modo.
   movlw S2
   call spause       ;Esperar aprox. 2 segundos antes de contestar
   
   ;Pedimos clave de acceso de 4 digitos. y damos 3 oportunidades.   
   movlw 3
   movwf cx                ;cx = 3
           
for_passwd:
      movlw ISD_INGRESAR   
      call play_isd        ;reproducir "INGRESAR"
      movlw ISD_CLAVE
      call play_isd        ;reproducir "CLAVE"
      
      movlw 4        
      call get_dtmfstr     ;Tomar 4 digitos DTMF y almacenar en AX:BX.
      movlw EE_PASSW_0     ;Comprobamos password.
      call eeprom_read 
      xorwf ax, W          
      btfss STATUS, Z      ;ax==EE_PASSW_0? (digito 3 y 2)
         goto wrong_pass   ;No, passwd incorrecto.
      movlw EE_PASSW_1
      call eeprom_read
      xorwf bx, W
      btfss STATUS, Z      ;bx==EE_PASSW_1?  (digito 1 y 0)
         goto wrong_pass   ;No, passwd incorrecto.
      goto end_for_passwd  ;Passwd Correcto.

wrong_pass:
         movlw ISD_CLAVE
         call play_isd        ;"CLAVE"
         movlw ISD_INCORRECTO
         call play_isd        ;"INCORRECTO"

         decfsz cx, F         ;Pedir de nuevo clave?
            goto for_passwd   ;Si. Uhh :)
end_for_passwd:

   movf cx, F
   btfsc STATUS, Z     ;Desconectar? cx==0?
      call close_call  ;Si, Clave ingresada mal 3 veces, desconectar :(

   bcf UCT_OPTION, QTIMEOUT      ;QTIMEOUT=0
   movlw S2
   call getdtmf                  ;Esperar 2 segundos para DTMF_0.
   bsf UCT_OPTION, QTIMEOUT      ;QTIMEOUT=1

   xorlw 0x00              
   btfsc STATUS, Z               ;DTMF_0 recibido?
      bsf UCT_OPTION, SKIPMENU   ;Si, UCT_OPTION/SKIPMENU=1
   movlw ISD_BIENVENIDO
   call play_isd                 ;"BIENVENIDO"

;*********************************************************************                  
;* uct_main_menu: Rutina de Menu principal.                          *
;*********************************************************************               
uct_main_menu:

   btfsc UCT_OPTION, SKIPMENU    ;SKIPMENU==1?
      goto get_menu_option       ;Si, omitir mensages del main_menu.
   movlw ISD_UNO
   call play_isd           ;"UNO"
   movlw ISD_CONTROL_DE
   call play_isd           ;"CONTROL DE" 
   movlw ISD_DISPOSITIVO
   call play_isd           ;"DISPOSITIVO" 
   movlw ISD_DOS
   call play_isd           ;"DOS"
   movlw ISD_VER_ESTADO_DE
   call play_isd           ;"VER ESTADO DE" 
   movlw ISD_DISPOSITIVO
   call play_isd           ;"DISPOSITIVO" 
   movlw ISD_TRES
   call play_isd           ;"TRES"
   movlw ISD_BUS_DCE
   call play_isd           ;"BUS_DCE" 
   movlw ISD_CUATRO
   call play_isd           ;"CUATRO" 
   movlw ISD_CONFIGURAR
   call play_isd
   movlw ISD_RINGS
   call play_isd           ;"CONFIGURAR RINGS"
   movlw ISD_CINCO
   call play_isd           ;"CINCO" 
   movlw ISD_CAMBIAR
   call play_isd           ;"CAMBIAR" 
   movlw ISD_CLAVE
   call play_isd           ;"CLAVE" 
   call play_sal_rep       ;"NUMERAL SALIR, ASTERISCO REPETIR".
   
   ;Tomamos DTMF y ejecutamos lo que corresponda.
get_menu_option:
   movlw S60                  ;Tiempo de espera maximo para getdtmf.
   call getdtmf
   movwf ax                   ;ax=DTMF recibido.

   movf ax, F
   btfsc STATUS, Z            ;ax==DTMF_0 ?
      goto skip_menu_msg      ;Si, des/activar UCT_OPTION/SKIPMENU
   movlw 0x01   
   xorwf ax, W 
   btfsc STATUS, Z            ;ax==DTMF_1 ?
      goto ctrl_disp_menu     ;Si, ir a control de dispositivos.
   movlw 0x02   
   xorwf ax, W
   btfsc STATUS, Z            ;ax==DTMF_2 ?
      goto disp_stat_menu     ;Si, ver estado de dispositivos.
   movlw 0x03   
   xorwf ax, W
   btfsc STATUS, Z            ;ax==DTMF_3 ?
      goto bus_dce_menu       ;Si, ir a BUS DCE.
   movlw 0x04   
   xorwf ax, W 
   btfsc STATUS, Z            ;ax==DTMF_4 ?
      goto conf_ring_menu     ;Si, ir menu de configurar RINGS.
   movlw 0x05   
   xorwf ax, W 
   btfsc STATUS, Z            ;ax==DTMF_5 ?
      goto chpassw_menu       ;Si, ir a menu de cambio de clave.
   movlw DTMF_AST   
   xorwf ax, W 
   btfsc STATUS, Z            ;ax==DTMF_* ?
      goto uct_main_menu      ;Si, repetir main menu.
   movlw DTMF_NUM   
   xorwf ax, W 
   btfsc STATUS, Z            ;ax==DTMF_# ?
      call close_call         ;Si, desconectar UCT.

   call play_incorrecto       ;"NUMERO INCORRECTO" 
   goto uct_main_menu         ;Volver a reproducir el menu principal.

;*********************************************************************               
;* skip_menu_msg: des/activar UCT_OPTION/SKIPMENU.                   *
;*********************************************************************            
skip_menu_msg:
      btfsc UCT_OPTION, SKIPMENU    ;SKIPMENU==0? Desactivado?
         goto skipmenu_off          ;No, desactivar.
      bsf UCT_OPTION, SKIPMENU      ;Si, activar.
      goto uct_main_menu
skipmenu_off:
      bcf UCT_OPTION, SKIPMENU      ;Desactivar.
      goto uct_main_menu

;*********************************************************************            
;* ctrl_disp_menu: menu de control de dispositivos ON/OFF.           *
;*********************************************************************            
ctrl_disp_menu:
      btfsc UCT_OPTION, SKIPMENU    ;SKIPMENU==1?
         goto get_disp              ;Si, omitir descripcion del menu.
      movlw ISD_INGRESAR
      call play_isd                 ;"INGRESAR"
      movlw ISD_DISPOSITIVO
      call play_isd                 ;"DISPOSITIVO"
      
get_disp:                     ;Esperamos numero de dispositivo.
      movlw S60
      call getdtmf
      movwf ax                ;ax=Numero de Dispositivo.

      movf ax, F
      btfsc STATUS, Z         ;ax==0 ?
         goto wrong_disp      ;Si, dispositivo incorrecto.
      movlw 5
      subwf ax, W             ;w=ax-5
      btfss STATUS, C         ;ax < 5 ?
         goto on_off_disp     ;Si, des/activar dispositivo.      
      movlw DTMF_AST
      xorwf ax, W
      btfsc STATUS, Z         ;ax==DTMF_* ?
         goto ctrl_disp_menu  ;Si, repetir menu. 
      movlw DTMF_NUM 
      xorwf ax, W 
      btfsc STATUS, Z         ;ax==DTMF_# ?
         goto uct_main_menu   ;Si, volver a menu principal.
      
wrong_disp:   
      call play_incorrecto    ;"NUMERO INCORRECTO"
      goto ctrl_disp_menu     ;Repetir menu.

on_off_disp:                  
      movlw S60
      call getdtmf            ;Esperamos accion a tomar, 
      movwf bx                ;activar o desactivar dispositivo?
      
      movlw 0x01
      xorwf bx, W
      btfsc STATUS, Z         ;bx==DTMF_1 ?
         goto on_disp         ;Si, activar dispositivo.
      movf bx, F
      btfsc STATUS, Z         ;bx==DTMF_0 ?
         goto off_disp        ;Si, desactivar dispositivo.
      movlw DTMF_NUM
      xorwf bx, W
      btfsc STATUS, Z         ;bx==DTMF_# ?
         goto ctrl_disp_menu  ;Si, volver a pedir dispositivo.
      movlw DTMF_AST
      xorwf bx, W
      btfsc STATUS, Z         ;bx==DTMF_* ? (No tiene sentido repetir)
         goto ctrl_disp_menu  ;Si, volver a pedir dispositivo.
      
      call play_incorrecto    ;"NUMERO INCORRECTO"
      goto on_off_disp        ;Pedir de vuelta accion.

on_disp:
         bsf ax, 7               
         movf ax, W              ;w=1000xxxx
         call ctrl_disp          ;activar dispositivo.
         movlw ISD_DISPOSITIVO
         call play_isd           ;"DISPOSITIVO"
         movlw ISD_ACTIVADO
         call play_isd           ;"ACTIVADO"
         goto ctrl_disp_menu     ;Repetir Menu.
   
off_disp:
         movf ax, W              ;w=0000xxxx
         call ctrl_disp          ;desactivar dispositivo.
         movlw ISD_DISPOSITIVO
         call play_isd           ;"DISPOSITIVO"
         movlw ISD_DESACTIVADO
         call play_isd           ;"DESACTIVADO"
         goto ctrl_disp_menu     ;Repetir Menu.
    
;*********************************************************************            
;* disp_stat_menu: muestra estado de los dispositivos ON/OFF         *   
;* conectados.                                                       *
;*********************************************************************            
disp_stat_menu:    
      movlw EE_DISPSTAT
      call eeprom_read
      movwf ax                   ;ax=estado de dispositivos.

      movlw ISD_DISPOSITIVO      
      call play_isd              ;"DISPOSITIVO"   

      movlw ISD_UNO              
      call play_isd              ;"UNO"
      btfss ax, 0                ;Disp. 1 activado ?
      goto d1_off                ;No.
         movlw ISD_ACTIVADO      ;Si.
         call play_isd           ;"ACTIVADO"
         goto d1_end
d1_off:
         movlw ISD_DESACTIVADO
         call play_isd           ;"DESACTIVADO"
d1_end:

      movlw ISD_DOS              
      call play_isd              ;"DOS"
      btfss ax, 1                ;Disp. 2 activado?
      goto d2_off                ;No.
         movlw ISD_ACTIVADO      ;Si.
         call play_isd           ;"ACTIVADO"
         goto d2_end
d2_off:
         movlw ISD_DESACTIVADO   
         call play_isd           ;"DESACTIVADO"
d2_end:

      movlw ISD_TRES              
      call play_isd              ;"TRES"
      btfss ax, 2                ;Disp. 3 activado?
      goto d3_off                ;No.
         movlw ISD_ACTIVADO      ;Si.
         call play_isd           ;"ACTIVADO"
         goto d3_end
d3_off:
         movlw ISD_DESACTIVADO   
         call play_isd           ;"DESACTIVADO"
d3_end:

      movlw ISD_CUATRO              
      call play_isd              ;"CUATRO"
      btfss ax, 3                ;Disp. 4 activado?
      goto d4_off                ;No.
         movlw ISD_ACTIVADO      ;Si.
         call play_isd           ;"ACTIVADO"
         goto d4_end
d4_off:
         movlw ISD_DESACTIVADO   
         call play_isd           ;"DESACTIVADO"
d4_end:
      goto uct_main_menu         ;Volver a menu Princial.

;*********************************************************************            
;* bus_dce_menu: Menu para interactuar con un dispositivo conectado  *  
;* al bus dce.                                                       *
;* OPCION UNO: Desactivada. Protocolo no terminado aun.              *
;*********************************************************************            
bus_dce_menu:
      btfsc UCT_OPTION, SKIPMENU    ;SKIPMENU==1?
         goto get_bus_dce_opt       ;Si, omitir descripcion del menu.
      movlw ISD_UNO
      call play_isd                 ;"UNO"
      movlw ISD_CONECTAR
      call play_isd                 ;"CONECTAR"
      movlw ISD_DOS             
      call play_isd                 ;"DOS"
      movlw ISD_DISPOSITIVO
      call play_isd                 ;"DISPOSITIVO"
      movlw ISD_EXTERNO
      call play_isd                 ;"EXTERNO"
     
get_bus_dce_opt:
      movlw S60
      call getdtmf
      movwf ax                ;ax=opcion.

      movlw 0x01
      xorwf ax, W   
      btfsc STATUS, Z         ;ax == 1?
         goto dce_connect     ;Si, Conectar con DCE.
      movlw 0x02
      xorwf ax, W   
      btfsc STATUS, Z         ;ax == 2?
         goto send_dtmf2dce   ;Si, enviar codigos DTMF del 0-9 al DCE.  
      movlw DTMF_AST
      xorwf ax, W
      btfsc STATUS, Z         ;ax = DTMF_*?
         goto  bus_dce_menu   ;Si, Repetir Menu.  
      movlw DTMF_NUM
      xorwf ax, W
      btfsc STATUS, Z         ;ax = DTMF_#?
         goto uct_main_menu   ;Si, Volver a menu Princial. 

;*********************************************************************            
;* dce_connect: Conecta con un dispositivo usando el bus DCE.        *   
;* La conexion se efectua mediante un protocolo.                     *
;* Esta funcionalidad NO esta implementada aun en el UCT.            *
;*********************************************************************            
dce_connect:
      movlw ISD_ERROR     ;Reproducir error y volver a menu anterior.
      call play_isd
      movlw ISD_CONECTAR
      call play_isd
      movlw ISD_BUS_DCE
      call play_isd
      goto bus_dce_menu   

;*********************************************************************            
;* send_dtmf2dce:                                                    *
;* Solo envia los codigos binarios DTMF del 0 al 9 al bus DCE, que   *
;* el cliente remoto solicite.                                       *
;* En este modo solo hay conexion unidireccional desde el UCT        *
;* al DCE.                                                           *
;*********************************************************************            
send_dtmf2dce:
      btfsc UCT_OPTION, SKIPMENU    ;SKIPMENU==1?
         goto get_dtmf_code         ;Si, omitir descripcion del menu.
      movlw ISD_INGRESAR
      call play_isd                 ;"INGRESAR"
      movlw ISD_NUMERO
      call play_isd                 ;"NUMERO"

get_dtmf_code:
      movlw M04                ;Esperar codigo DTMF por 4 minutos.
      call getdtmf
      movwf ax                ;ax=codigo dtmf.

      movlw DTMF_AST
      xorwf ax, W
      btfsc STATUS, Z         ;ax = DTMF_*?
         goto  send_dtmf2dce  ;Si, Repetir Menu.  
      movlw DTMF_NUM
      xorwf ax, W
      btfsc STATUS, Z         ;ax = DTMF_#?
         goto bus_dce_menu    ;Si, Volver a menu anterior.

      ;Enviamos codigo DTMF al DCE.
   
      movf ax, W                 ;W=ax
      call bdce_write            ;Escribimos en bus DCE y nos fijamos
      xorlw ERR                  ;si la operacion fue exitosa.
      btfsc STATUS, Z            ;W==ERR?
         goto send_dtmf2dce_err  ;Si, error al escribir dato.
      movlw ISD_LISTO            ;No, dato escrito con exito.
      call play_isd              ;"LISTO"
      goto send_dtmf2dce         ;Volver a pedir dtfm.

send_dtmf2dce_err:
      movlw ISD_BUS_DCE
      call play_isd              ;"BUS DCE"
      movlw ISD_ERROR
      call play_isd              ;"ERROR"
      goto send_dtmf2dce         ;Volver a pedir dtfm.

;*********************************************************************            
;* conf_ring_menu: Menu para configurar el numero de rings que       *
;* deben pasar antes del que UCT conteste un llamado.                *
;* Si el usuario ingresa DTMF_0, EE_RINGS se establece a _LOT_RINGS. *
;*********************************************************************            
conf_ring_menu:
      btfsc UCT_OPTION, SKIPMENU    ;SKIPMENU==1?
         goto get_rings             ;Si, omitir descripcion del menu.
      movlw ISD_INGRESAR
      call play_isd                 ;"INGRESAR"
      movlw ISD_RINGS
      call play_isd                 ;"RINGS"
      
get_rings:                    ;Esperamos cantidad de RINGS.
      movlw S60
      call getdtmf
      movwf ax                ;ax=Numero de RINGS.

      movf ax, F
      btfsc STATUS, Z         ;ax == 0?
         goto set_LOT_RINGS   ;Si, setar EE_RINGS = _LOT_RINGS
      movlw DTMF_AST
      xorwf ax, W
      btfsc STATUS, Z         ;ax = DTMF_*?
         goto  conf_ring_menu ;Si, Repetir Menu.  
      movlw DTMF_NUM
      xorwf ax, W
      btfsc STATUS, Z         ;ax = DTMF_#?
         goto uct_main_menu   ;Si, Volver a menu Princial. 

      goto set_EE_RINGS       ;No, Setear EE_RINGS = ax.
           
set_LOT_RINGS:
      movlw _LOT_RINGS
      movwf ax                ;ax = _LOT_RINGS

set_EE_RINGS:
      movlw EE_RINGS
      movwf ex
      movf ax, W
      call eeprom_write       ;EE_RINGS = ax
      
      movlw ISD_LISTO
      call play_isd           ;"LISTO"

      goto uct_main_menu      ;Volver a menu Princial. 
         
;*********************************************************************            
;* chpassw_menu: Menu para cambiar clave de acceso en el UCT.        *
;* Por seguridad, primero debe ingresar DTMF UNO, y luego la         *
;* nueva clave.                                                      *
;*********************************************************************            

chpassw_menu:
      btfsc UCT_OPTION, SKIPMENU      ;SKIPMENU==1?
         goto get_chpassw_menu_option ;Si, omitir descripcion.
      movlw ISD_UNO
      call play_isd                 ;"UNO"
      movlw ISD_CAMBIAR
      call play_isd                 ;"CAMBIAR"
      movlw ISD_CLAVE
      call play_isd                 ;"CLAVE"
      call play_sal_rep             ;"# SALIR, * REPETIR"
      
      
get_chpassw_menu_option:      ;Esperamos opcion.
      movlw S60
      call getdtmf
      movwf ax                ;ax=opcion.

      movlw 0x01
      xorwf ax, W
      btfsc STATUS, Z         ;ax == 1?
         goto set_passwd      ;Si, setear nuevo password.
      movlw DTMF_AST
      xorwf ax, W
      btfsc STATUS, Z         ;ax = DTMF_*?
         goto chpassw_menu    ;Si, Repetir Menu.  
      movlw DTMF_NUM
      xorwf ax, W
      btfsc STATUS, Z         ;ax = DTMF_#?
         goto uct_main_menu   ;Si, Volver a menu Princial. 

      call play_incorrecto
      goto chpassw_menu       ;Volver a repetir Menu.
     
      
set_passwd:
      movlw ISD_INGRESAR
      call play_isd           ;"INGRESAR"
      movlw ISD_CLAVE         
      call play_isd           ;"CLAVE"

      movlw 0x04              ;Tomar 4 digitos DTMF y almacenar en
      call get_dtmfstr        ;AX:BX.

      movlw EE_PASSW_0        ;EE_PASSWD_0 = Digito 3 y 4
      movwf ex
      movf ax, W
      call eeprom_write       ;EE_PASSWD_0 = ax
      
      movlw EE_PASSW_1        ;EE_PASSWD_1 = Digito 1 y 2
      movwf ex
      movf bx, W
      call eeprom_write       ;EE_PASSWD_1 = bx

      movlw ISD_LISTO
      call play_isd           ;"LISTO"

      goto uct_main_menu      ;Volver a menu Princial. 

;*********************************************************************            
;*                       RUTINAS DEL BUS DCE DEL UCT                 *
;*********************************************************************            
;* NOTA: Solo es posible enviar datos binarios del 0 al 9 por el     * 
;*       bus DCE, pero no recibir datos, ni usar el protocolo CTP.   *
;*       El bus DCE aun no esta totalmente implementado.             *
;*********************************************************************            

;*********************************************************************            
;* Rutinas del PROTOCOLO CTP (Command Transmision Protocol)          *
;* NO DEFINIDO! - NO TERMINADAS.                                     *
;*********************************************************************                    

;*********************************************************************            
;* RUTINAS DEL UCT PARA COMUNICARSE POR EL BUS DCE                   *
;*********************************************************************               

;*********************************************************************            
;* bdce_write(): escribe el nibble menos significativo               *
;* de W en el BUS DCE.                                               *
;* I:                                                                *
;*    W  = valor a escribir en el bus DCE.                           *
;* O:                                                                *
;*    W = OK  -> El dato fue escrito en el bus DCE.                  *
;*    W = ERR -> El BUS DCE esta ocupado, dato no se escribe.        *
;* Destruye: ex, W.                                                  *
;* NOTA:                                                             *
;* 1) El dato se mantiene 5 ms en el BUS DCE y DCEINT_PIN se         *
;*    mantiene a 0 por el mismo periodo. Luego el bus queda en       *   
;*    alta Z y DCEINT_PIN=1.                                         *   
;*********************************************************************            
bdce_write:

   movwf ex             ;ex=W

   BANK_1
   movlw 0x0F
   andwf TRISB, F       ;TRISB=0000xxxx -> RB4-7=S
   BANK_0

   ;Preparamos datos para escribir en el bus DCE.
   movlw 0x0F
   andwf PORTB, F       ;PORTB=0000xxxx
   swapf ex, F          ;ex=swap(ex)
   movlw 0xF0
   andwf ex, W          ;W=xxxx0000=AND(0xF0,ex)
   iorwf PORTB, F       ;Escribimos dato en RB4-7 del PORTB.

   ;Comprobamos si se puede escribir dato en BUS DCE.
   btfss UCTINT_PIN     ;Bus DCE ocupado? UCTINT_PIN == 0? 
      retlw ERR         ;Si, retornar con W=ERR.
   
   ;Escribir dato.
   bsf PORTA, 3         ;Si, Habilitar 4016 (sacar de alta Z).
   bcf DCEINT_PIN       ;Ocupamos BUS DCE.

   movlw d'5'
   call mpause          ;Mantener dato por 5 ms.

   bsf DCEINT_PIN       ;Desocupamos BUS DCE. -> DCEINT_PIN=1.
   bcf PORTA, 3         ;Deshabilitar 4016 (poner en alta Z).

   movlw 0x0F
   andwf PORTB, F       ;PORTB=0000XXXX, Bus de datos a CERO.
   
   retlw OK             ;Ecritura finalizada, retornar con W=OK.

;*********************************************************************            
;* dce_connection: establece conexion con DCE.                       *   
;* Funcion no habilitada.                                            *
;*********************************************************************            
dce_connection:
   goto RESET           ;Resetear en caso de interrupcion por DCE.
;NO DEFINIDO!

;*********************************************************************            
;*                    RUTINAS GENERALES DEL UCT                      *
;*********************************************************************        

;*********************************************************************            
;* close_call(): Desconecta al UCT de la llamada actual.             *
;*********************************************************************               
close_call:

   btfss UCT_OPTION, QMSG  ;Reproducir mensages antes de salir?
      goto end_close_call  ;No, salir sigilosamente...
   movlw ISD_SALIR         ;Si, "SALIR".
   call play_isd           ;antes de desconectar.
   movlw S2
   call spause             ;Esperar 2 seg. entes de RESETEAR.

end_close_call:
   goto RESET              ;Good Nigth!

;*********************************************************************            
;* get_dtmfstr(): Toma una cadena de 4 codigos DTMF como maximo,     *
;* usando la funcion getdtmf().                                      *
;* I: W = Numero de codigos DTMF a tomar (max. 4).                   *
;* O: AX y BX almacenan los 4 codigo DTMF recibidos. Donde el nibble *
;* menos significativo de BX contiene el ultimo codigo DTMF recibido.* 
;* Si W < 4 los codigos DTMF faltantes en AX y BX se completan con   *
;* cero. TIMEOUT en 30 segundos si no se recive ningun codigo DTMF.  *
;* Destruye: W                                                       *   
;* Registros stack-eados: ax, cx, dx, ex                             *
;*********************************************************************            
get_dtmfstr:

   ;ax y ex se stakea en getdtmf.
   call push_cx
   call push_dx
      
   clrf ax
   clrf bx                   ;BX y AX Almacenan DMTF's recibido.
   movwf cx                  ;Numero de digitos DTMF a tomar. 
for_get_dtmfstr:
   movlw S60                 ;timeout en 60 segundos para getdtmf
   call getdtmf              ;W = codigo DTMF recibido. W=0000XXXX
   addwf bx, F               ;bx=DTMF
   decfsz cx, F              ;Tomar otro codigo DTMF?
   goto get_next_dtmfstr     ;Si!
   goto end_for_get_dtmfstr  ;No! cx==0.   

get_next_dtmfstr:        ;Preparo [ax:bx] para recibir proximo DTMF.
   movlw 0x04
   movwf dx              ;dx=4 numero de rotaciones.
rotate_4left:            ;Desplazo [ax:bx] 4 lugares a la izquierda.         
      bcf STATUS, C      ;Limpio CARRY, en proxima rotacion meto C=0.
      rlf bx, F          ;Roto izq. bx y 7 bit se pone en CARRY.
      rlf ax, F          ;Roto izq. ax e introdusco CARRY anterior.
      decfsz dx, F
      goto rotate_4left
   goto for_get_dtmfstr  ;Busco proximo DMTF!

end_for_get_dtmfstr:     ;No hay mas DTMF que obtener.

   call pop_cx
   call pop_dx
   
   return               ;back to caller!

;*********************************************************************            
;* play_incorrecto(): reproduce "NUMERO INCORRECTO".                 *   
;* Destruye: W                                                       *
;*********************************************************************            
play_incorrecto:
   movlw ISD_NUMERO
   call play_isd              ;"NUMERO"
   movlw ISD_INCORRECTO       
   call play_isd              ;"INCORRECTO"
   return   

;*********************************************************************            
;* play_rep-sal(): reproduce "NUMERAL SALIR, ASTERISCO REPETIR".     *
;* Destruye: W                                                       *
;*********************************************************************            
play_sal_rep:   
   movlw ISD_NUMERAL
   call play_isd              ;"NUMERAL"
   movlw ISD_SALIR
   call play_isd              ;"SALIR"
   movlw ISD_ASTERISCO
   call play_isd              ;"ASTERISCO"
   movlw ISD_REPETIR
   call play_isd              ;"REPETIR"
   return   

;*********************************************************************            
;* mpause(): establece una pausa de W mili-segundos en el programa.  *    
;* La pausa es de: [((1Mhz/8)/125)]^(-1) * W = 1 ms * W .            *      
;* I: W = Multiplicador o factor .                                   *
;* Destruye: W, ex.                                                  *
;* Nota:                                                             *
;* 1) Desactiva interrupcion Por TMR0 y modifica OPTION_REG.         *
;* 2) No produce interrupciones.                                     *
;* 3) Si UCT_OPTION\QF_DTFM==1 y si hay un codigo DTMF presente la   *
;*    funcion retorna imediatamente.                                 *
;*********************************************************************               
mpause:

   movwf ex                ;ex=W -> Multiplicador.

   BANK_1
   bcf OPTION_REG, T0CS    ;T0CS=0 -> Usar clock interno.
   bcf OPTION_REG, PSA     ;PSA=0 -> Prescaler asignado a TMR0.
   bcf OPTION_REG, PS2     ;PRESCALER RATE: 1:8  
   bsf OPTION_REG, PS1  
   bcf OPTION_REG, PS0  

   BANK_0
   bcf INTCON, T0IE        ;T0IE=0 -> no interrupcion por TMR0. 
   bcf INTCON, T0IF        ;TOIF=0 -> Ponemos Flag a cero. 

wait_mpause:   
   clrwdt
   movlw 0x83              ;W=neg(0x7D)=neg(d'125')=0x83
   movwf TMR0              ;Cargamos TMR0=W

tmr0_overflow:

   btfss UCT_OPTION, QF_DTMF ;Salir si hay un Codigo DTMF presente?
   goto no_dtmf_chk          ;No. Ignorar comprobacion de DTMF.
   btfsc PORTA, 4            ;Hay un codigo DTMF presente en el 8870 ?
   return                    ;Si. Salir de funcion.

no_dtmf_chk:
   btfss INTCON, T0IF      ;Desbordo el TMR0?
      goto tmr0_overflow   ;No, Seguir esperando.
   bcf INTCON, T0IF        ;Si, Reponemos flag.   
   
   decfsz ex, F            ;ex == 0?
      goto wait_mpause     ;No, seguir esperando.
   return                  ;Si, volver.

;*********************************************************************            
;* spause(): genera una pausa de W segundos en el programa.          *   
;* I: W = Pausa de N segundos.                                       *
;*********************************************************************            
spause:

   bcf UCT_OPTION, QTIMEOUT ;No resetear al producirse un TIMEOUT
   call set_timeout         ;Producir un TIMEOUT en W segundos.

wait_TIMEOUT:
   clrwdt
   btfsc UCT_STATUS, TIMEOUT  ;Ocurrio un TIMEOUT==1?
      goto end_spause         ;Si, volver.
   goto  wait_TIMEOUT         ;No, seguir esperando.
 
end_spause:
   call unset_timeout         ;Desactivar interrupcion por TMR0.
   bsf UCT_OPTION, QTIMEOUT   ;QTIMEOUT siempre debe estar activada.
   return                     ;back to caller.

;*********************************************************************            
;* set_timeout(): setea el temporizador para que use clock interno   *   
;* del pic16f84a y produce un TIMEOUT en W segundos.                 *
;*                                                                   *
;* I: W = tiempo en segundos antes de producir el TIMEOUT.           *
;* Modifica: SECONDS, TMR0_AUX, TMR0 y OPTION_REG.                   *
;*                                                                   *
;* Notas:                                                            *
;* 1) Se debe usar unset_timeout() para desactivar esta funcion.     *
;* 2) GIE debe estar habilitado y para desabilitar esta funcion      *
;* se debe inhibir el flag de interrupcion por TMR0. Para ello usar: *
;* call unset_timeout -> desactiva un TIMEOUT seteado por            *
;* set_timeout().                                                    *
;*                                                                   *
;* Funcionamiento:                                                   *
;* La frecuencia del clock interno sera fi=4Mhz/4 = 1 Mhz .          *
;* Luego el prescaler divide por 32, fi/32= 31250 Hz. E incrementara *
;* al TMR0 cada 32 pulsos de clock.                                  *
;* El registro TMR0 se carga con TMR0=6, por lo tanto                *
;* se producira una interrupcion cada 250 incrementos de TMR0.       *
;* Esto es: (fi/32)/250 = 125 Hz.                                    *
;* Posteriormente se carga la variable TMR0_AUX con 125, y se la     *
;* decrementa cada 125 interrupciones del timer.                     *
;* Quedando: ((fi/32)/250)/125 = 1 HZ. Lo que equivale a 1 segundo.  *
;* Finalmente se decrementa la variable SECONDS y si resulta cero    *
;* activa el flag TIMEOUT de la variable UCT_STATUS.                 *
;* Luego si flag QTIMEOUT esta activada en UCT_OPTION el UCT se      *
;* reiniciara.                                                       *
;* El encargado de manegar cada interrupcion es la funcion RSITMR0 y *
;* decide que hacer con dicha interrupcion.                          *
;*********************************************************************            
set_timeout:

   BANK_1   
   bcf OPTION_REG, T0CS ;Usar fuente de clock interna, 1 MHz.
   bcf OPTION_REG, PSA  ;Prescaler asignado al TMR0.
   bsf OPTION_REG, PS2  ;Prescaler divide frec. de clock por 32.
   bcf OPTION_REG, PS1  ;Es decir cada 32 pulsos de clock se incremen-
   bcf OPTION_REG, PS0  ;tara en 1 el registro TMR0.
   BANK_0

   movwf SECONDS        ;Numero de segundos para que ocurra el TIMEOUT.
   movlw d'125'         ;Decrementar SECONDS cada 125 interrupciones
   movwf TMR0_AUX       ;del timer 0.
   movlw 0x06           ;Producir una interrupcion cada 250 cuentas
   movwf TMR0           ;o incrementos del TMR0
   bsf INTCON, T0IE     ;habilitar interrupcion por TMR0
   return               ;back to caller!

;*********************************************************************            
;* unset_timeout(): desactiva un TIMEOUT y asigna prescaler al       *   
;* watchdog.                                                         *
;* Repone el flag TIMEOUT a cero.                                    *
;*********************************************************************            
unset_timeout: 
   bcf UCT_STATUS, TIMEOUT ;Nos aseguramos que TIMEOUT=0.
   bcf INTCON, T0IE        ;desactivar interrupcion por TMR0.
   BANK_1
   bsf OPTION_REG, PSA
   BANK_0
   return

;*********************************************************************            
;                RUTINAS DE MANEJO DE CIRCUITOS DEL UCT              *
;*********************************************************************        

;*********************************************************************            
;* ctrl_disp(): activa o desactiva dispositivo externo.              *   
;* I: W=S000xxxx                                                     *
;* Donde si S: 1 -> activa dispositivo 0 -> desactiva dispositivo.   *
;* y xxxx representa el numero de dispositivo a activar.             *
;* Destruye: cx, dx, ex,W                                            *
;* NOTA:                                                             *
;* Si el numero de dispositivo es distinto de [1,2,3,4], solo        *
;* actualiza el latch 4042, con el estado de los disp. almacenado en *
;* EEPROM. Util para inicializar dispositivos al comienzo del UCT.   *
;*********************************************************************            
ctrl_disp:
   
   movwf ex       ;Dispositivo a activar.

   BANK_1
   ;Configuramos BUS de datos RB4-7 entre dispositivos.
   ;RB4-7=S -> conectados a latch 4042.
   movlw 0x0F
   andwf TRISB, F ;TRISB=0000XXXX

   BANK_0
   clrf cx        ;cx bit-0 = Tipo de operacion. 1=Activar/0=Desac.
   btfsc ex, 7    ;Activar disp. -> S=1?
      bsf cx, 0   ;Si activar dispositivo.

   bcf ex, 7      ;Borramos bit-7 de ex.

   movlw EE_DISPSTAT    ;EE_DISPSTAT=0000XXXX
   call eeprom_read
   movwf dx             ;Estado de dispositivos ON/OFF actual.
      
   ;Determinamos que dispositivo activar o desactivar.
   ;El nuevo estado de los disp. ON/OFF queda en dx.

;if_d4  
   movlw 4
   xorwf ex, W
   btfss STATUS, Z            ; ex==4, Dispositivo 4 ?
   goto else_if_d3            ; No.
      btfss cx, 0             ; Activar?
      goto d4_turn_off        ; No.
         bsf dx, 3            ; D4 Activado.
         goto end_if_switch
d4_turn_off:
         bcf dx, 3            ; D4 Desactivado.
         goto end_if_switch

else_if_d3:
   movlw 3
   xorwf ex, W
   btfss STATUS, Z            ; ex==3, Dispositivo 3 ?
   goto else_if_d2            ; No.
      btfss cx, 0             ; Activar?
      goto d3_turn_off        ; No.
         bsf dx, 2            ; D3 Activado.
         goto end_if_switch
d3_turn_off:
         bcf dx, 2            ; D3 Desactivado.
         goto end_if_switch

else_if_d2:
   movlw 2
   xorwf ex, W
   btfss STATUS, Z            ; ex==2, Dispositivo 2 ?
   goto else_if_d1            ; No.
      btfss cx, 0             ; Activar?
      goto d2_turn_off        ; No.
         bsf dx, 1            ; D2 Activado.
         goto end_if_switch
d2_turn_off:
         bcf dx, 1            ; D2 Desactivado.
         goto end_if_switch

else_if_d1:
   movlw 1
   xorwf ex, W
   btfss STATUS, Z            ; ex==1, Dispositivo 1 ?
   goto end_if_switch         ; No, solo actualizar latch 4042. 
      btfss cx, 0             ; Activar?
      goto d1_turn_off        ; No.
         bsf dx, 0            ; D1 Activado.
         goto end_if_switch
d1_turn_off:
         bcf dx, 0            ; D1 Desactivado.
         goto end_if_switch
   
end_if_switch:
   
   ;Grabamos estado de los dispositivos en el memoria EEPROM.
   ;Si el nuevo estado de disp. es igual al anterior, no
   ;grabamos en la mem. eeprom, pero si actualizamos latch 4042.
   movlw EE_DISPSTAT
   call eeprom_read
   xorwf dx, W
   btfsc STATUS, Z           ;dx == EE_DISPSTAT ?
   goto latch_update         ;Si. Solo actualizar latch.
      movlw EE_DISPSTAT      ;No. Grabar en eeprom y actualizar latch.
      movwf ex               ;Direccion eeprom y
      movf dx, W             ;Dato a grabar.
      call eeprom_write

latch_update:   
   ;Ponemos estado de disp. en el BUS de datos RB4-7
   movlw 0x0F
   andwf PORTB, F             ; PORTB=0000xxxx
   swapf dx, F                ; dx=xxxx0000
   movf dx, W                 ; W=xxxx0000
   iorwf PORTB, F             ; PORTB[4-7]=dx
   
   ;Seleccionamos Latch 4042 para usar con BUS de datos RB4-7 y
   ;el estado de los dispositivos se transfieren al LATCH. 
   bsf PORTA, 2      ;RA2=1 -> activa entradas de 4042.
   
   nop               ;Esperamos tiempo de propagacion del 4042
   nop               ;del orden de nano-segundos.
   
   ;Desactivamos Latch 4042 del Bus de datos RB4-7 
   bcf PORTA, 2      ;RA2=0 -> desactiva entradas de 4042.

   movlw 0x0F
   andwf PORTB, F    ;PORTB=0000XXXX, Bus de datos a CERO.
      
end_ctrl_disp:
   return            ;back to caller!   

;*********************************************************************               
;* getdtmf(): Espera un codigo DTMF por un tiempo determinado        *   
;* almacenado en W. Si el codigo dtmf no se recibe en el             *
;* tiempo especificado en W, produce una intertupcion                *
;* que es manejada por RSITMR0. De lo contrario devuelve             *
;* en W el codigo DTMF recibido.                                     *
;* Si QTIMEOUT=0, y el tiempo de espera expira la funcion regresa    *
;* W=0xFF, como indicacion que ningun dtmf fue recibido.             *
;* I: W = time-out en segundos.                                      *
;* O: W = codigo DTMF recibido. (DMTF 0 se codifica como W=0x00)     *
;* Registros stack-eados: ax, ex                                     *
;*********************************************************************            
getdtmf:

   call push_ax      ; salvamos ax.
   call push_ex      ; salvamos ex.
   call set_timeout  ; seteamos un timeout en W segundos. 
   
wait_dtmf:
      clrwdt
      btfsc PORTA, 4        ;Hay un codigo DTMF presente en el 8870 ?
         goto dtmf_recived       ;Si, tomar codigo.
      btfss UCT_STATUS, TIMEOUT  ;Se produjo un TIMEOUT==1?
         goto wait_dtmf          ;No, seguir esperando.
      call unset_timeout         ;Si, sacar timeout y retornar 
      movlw 0xFF                 ;con W=0xFF.
      goto end_getdtmf           ;Salir de funcion.

dtmf_recived:
   ;Codigo DTMF recibido.
   call unset_timeout   ; No permitir timeout en W segundos.
         
   BANK_1
   ;Configuramos BUS de datos RB4-7 entre dispositivos como entradas.
   movlw 0xF0
   iorwf TRISB, F      ;RB4-7=Entradas -> conectado a Q1-4 de 8870.

   BANK_0
   ;Seleccionamos al decodificador 8870 para usar con BUS de datos.
   bsf PORTA, 0      ;RA0=1 -> saca de alta Z, Q1-Q4 de 8870.
   nop               ;2 uS para que Q1-Q4 salgan de alta impedancia.
   nop               ;(del orden de nano-segundos)

   ;Leemos codigo DTMF
   movf PORTB, W     ;En 8870 Leo Codigo DTMF de Q1-Q4
   andlw 0xF0        ;W=xxxx0000
   movwf ax          ;ax=W=xxxx0000
   swapf ax,F        ;ax=0000xxxx
   
   ;Quitamos 8870 del Bus de datos RB4-7
   bcf PORTA, 0      ;RA0=0 -> pone en alta impedancia Q1-Q4 de 8870.
   
   ;Dejamos Bus de datos RB4-7 conf. como salidas.
   BANK_1
   movlw 0x0F
   andwf TRISB, F    ;RB4-7=Salidas
   BANK_0            ;Dejamos banco cero (por convencion)
   movlw 0x0F
   andwf PORTB, F    ;PORTB=0000XXXX, Bus de datos a CERO.

   ;El 8870 codifica el DTMF 0 como 0x0A, si se recibe DTMF 0
   ;la funcion devolvera W=0. Para preservar naturalidad en numeros.
   movlw 0x0A        
   xorwf ax, W
   btfsc STATUS, Z   ;ax == 0x0A ?   
      clrf  ax       ;Si, hacer ax=0.

   movf ax, W        ;W = DTMF recibido.

   ;Esperamos que el usuario deje de enviar el mismo DTMF. 
wait_dtmf_end:
   clrwdt
   btfsc PORTA, 4      ;El codigo DTMF todavia esta presente?
   goto wait_dtmf_end  ;Si, esperar a que finalize.

end_getdtmf:
   call pop_ax       ;reponemos viejo valor de ax.
   call pop_ex       ;reponemos viejo valor de ex.
   return            ;back to caller!

;*********************************************************************            
;* play_isd(): Reproduce el mensaje numero W almacenado en la        * 
;* memoria ISD1420, retorna el control al programa cuando termina de *
;* reproducirse el mensaje.                                          *
;* I: W = Numero de mensage                                          *
;* Destruye: W.                                                      *
;* Registros stack-eados: ax y ex.                                   *
;* Nota:                                                             *   
;* 1) Si hay un codigo DTMF presente la funcion retorna imediatamente*
;*    sin reproducir el mensage de audio.                            *   
;* 2) Si el mensaje se esta reproduciendo, retorna imediatamente,    *
;*    pero el mensaje no sera cortado y seguira reproduciendoce      *
;*    hasta que la isd1420 encuente un EOM.                          *
;* 3) El retardo desde que detecta un DTMF y la funcion retorna,     *
;*    puede ser desde 4 uS a 110 uS.                                 *
;*********************************************************************            
play_isd:

   btfsc PORTA, 4    ;Hay un codigo DTMF presente en el 8870 ?
   return            ;Si. Salir de funcion.

   call push_ax
   call push_ex
   movwf ax          ;ax=W -> Numero de mensage.

   BANK_1
   ;Configuramos BUS de datos RB4-7 entre dispositivos.
   bcf TRISB, 4      ;RB4=S Conectado a A0 de ISD1420
   bcf TRISB, 5      ;RB5=S Conectado a /PLAYE de ISD1420
   bsf TRISB, 6      ;RB6=E Conectado a /RECLED de ISD1420
   bcf TRISB, 7      ;RB7=S Conectado a A4 de ISD1420

   BANK_0
   ;Valor inicial del BUS de datos RB4-7
   ;ISD1420: en modo MESSAGE CUEING + CONSECUTIVE ADDRESS
   movlw b'10110000'       ;RB7=RB5=RB4=1 => (A4=1,A0=1, /PLAYE=1)
   iorwf PORTB, F       

   ;Seleccionamos ISD1420 para usar con BUS de datos.
   bsf PORTA, 1      ;RA1=1 -> Saca de alta Z a 4016.
   nop               ;Esperamos Tiempos de propagacion.
   nop

   ;Permitimos que mpause() retorne imediatamente si hay un DTMF.
   bsf UCT_OPTION, QF_DTMF 

   ;Elegimos mensage a reproducir en ISD1420.
   decf ax, F           ;AX=W-1 (Ver principio de funcion)
loop_msg_select:
      bcf PORTB, 5      ;RB5=0 -> /PLAYE=0 pone puntero interno 
                        ;de ISD en siguiente mensage. 
      movlw d'20'
      call mpause       ;Esperar 20 mS.
      
      btfsc PORTA, 4    ;Hay un codigo DTMF presente en el 8870 ?
      goto end_play_isd ;Si. Salir de funcion.

      bsf PORTB, 5      ;RB5=1 -> /PLAYE=1
      movlw d'20'
      call mpause       ;Esperar 20 mS. Esperar Avance de Ptro. Interno.

      btfsc PORTA, 4    ;Hay un codigo DTMF presente en el 8870 ?
      goto end_play_isd ;Si. Salir de funcion.

      decfsz ax, F
      goto loop_msg_select

   ;Quitamos modo MESSAGE CUEING, pero dejamos CONSECUTIVE ADDRESS
   ;en ISD1420 para que no se resetee puntero interno.
   bcf PORTB, 4      ;RB4=0 -> A0=0 en ISD1420 
   movlw d'5'        ;Esperar 5 mS
   call mpause

   btfsc PORTA, 4    ;Hay un codigo DTMF presente en el 8870 ?
   goto end_play_isd ;Si. Salir de funcion.

   ;Reproducimos mensage   
   bcf PORTB, 5      ;RB5=0 -> /PLAYE=0 reproduce mensage seleccionado.
   movlw d'5'        ;Esperar 5 mS
   call mpause
   bsf PORTB, 5      ;RB5=1 -> /PLAYE=1

   ;Esperamos a que el mensage se termine de reproducir.
wait_play_msg:
      clrwdt
      btfsc PORTA, 4       ;Hay un codigo DTMF presente en el 8870 ?
      goto end_play_isd    ;Si. Salir de funcion.
      btfsc PORTB, 6       ;(RB6=/EOM) == 0?
      goto wait_play_msg   ;No, /EOM == 1. Seguir esperando.

end_play_isd:
   bcf PORTB, 7       ;RB7=0 -> A4=0 en ISD1420.
                      ;Reseteamos puntero interno de memoria de la ISD1420.

   ;Quitamos ISD1420 del Bus de datos RB4-7
   bcf PORTA, 1       ;RA1 -> pone en alta Z a 4016.

   ;Dejamos Bus de datos RB4-7 conf. como salidas.
   BANK_1
   bcf TRISB, 6       ;RB6=S
   movlw 0x0F
   andwf PORTB, F     ;PORTB=0000XXXX, Bus de datos a CERO.
   BANK_0             ;Dejamos banco cero (por convencion)

   movlw d'20'
   call mpause        ;Esperar 20 mS. Por si se usa play_isd() en loop.

   ;QF_DTFM=0. 
   bcf UCT_OPTION, QF_DTMF 

   call pop_ax
   call pop_ex
   return             ;back to caller!   

;*********************************************************************            
;              RUTINAS DE APOYO INDEPENDIENTES DEL UCT               *   
;*********************************************************************        

;*********************************************************************            
;* eeprom_read(): W=*(W)                                             *   
;* Regresa en W el contenido de una direccion de la memoria EEPROM   *
;* almacenado en W.                                                  *
;* I: W = direcccion de memoria EEPROM                               *   
;* O: W = valor de la variable en la direccion de memoria EEPROM.    *
;*********************************************************************            
eeprom_read:
   BANK_0
   movwf EEADR       ;Direccion a leer
   BANK_1
   bsf EECON1, RD    ;Leer EEPROM
   BANK_0
   movf EEDATA, W    ;W = EEDATA

   return            ;Back to caller!

;*********************************************************************            
;* eeprom_write(): Escribe el valor de W en la memoria EEPROM, en la * 
;* direccion contenida en ex.                                        *
;* I: W  = valor a escribir en la mem EEPROM.                        *
;*    ex = direccion de la mem EEPROM a escribir.                    *
;* Destruye: W                                                       *
;* Nota: bit GIE de INTCON es activado.                              *
;*********************************************************************            
eeprom_write:
   
   BANK_0
   movwf EEDATA
   movf ex, W
   movwf EEADR

   BANK_1
   bcf INTCON, GIE      ;impedir interrupciones.
   bsf EECON1, WREN     ;activar permiso de escritura de eeprom.
   movlw 0x55           ;start: secuencia de inicializacion.
   movwf EECON2
   movlw 0xAA
   movwf EECON2         ;end: secuencia de inicializacion.

   bsf EECON1, WR       ;empezar a escribir en eeprom.
   
wait_eeprom_write_time:
      clrwdt                        
      btfss EECON1, EEIF            ;escritura finalizada?
      goto wait_eeprom_write_time   ;No, seguir esperando. 

   bcf EECON1, EEIF     ;reponer flag.
   bcf EECON1, WREN     ;desactivar permiso de escritura de eeprom.

   bsf INTCON, GIE      ;activar interrupciones.
   BANK_0
   return               ;back to caller! 


;*********************************************************************            
;* MINI-STACK_SYSTEM:                                                *
;* Funciones para salvar y reponer registros de forma individual.    *     
;* Funciones pop_*() reponen un registro salvado previamente con     *
;* push_*().                                                         *
;* Donde * = [ax|bx|cx|dx|ex|w|STATUS]                               *
;* ATENCION: Usar dos veces concecutivas push_*() sobreescribira el  *
;* valor almacenado previamente con el primer push_*().              *
;* Por ello en cada funcion que use el stack debemos, especificar    *
;* que registro pone en el stack para no sobrescribir y perder       *
;* datos.                                                            *
;* NOTA:                                                             *
;* 1) Ninguna funcion modifica el registro W, excepto pop_w().       *
;* 2) Excepto push_w(), push_STATUS(), pop_w() y pop_STATUS(), todas *
;*    las funciones PUEDEN modificar el bit Z del registro STATUS.   *
;* AUTOR: Boris Estudiez.                                            *
;*********************************************************************            

;*********************************************************************            
;* pop_*(): Reponen un registro que fue almacenado con push_*().     *
;*********************************************************************            

pop_ax:
   movwf TEMPX       ;TEMPX=W
   movf TEMP_ax, W   ;W=TEMP_ax 
   movwf ax          ;ax=TEMP_ax=W
   movf TEMPX, W     ;W=TEMPX
   return
pop_bx:
   movwf TEMPX
   movf TEMP_bx, W    
   movwf bx
   movf TEMPX, W
   return
pop_cx:
   movwf TEMPX
   movf TEMP_cx, W    
   movwf cx
   movf TEMPX, W
   return
pop_dx:
   movwf TEMPX
   movf TEMP_dx, W    
   movwf dx
   movf TEMPX, W
   return
pop_ex:
   movwf TEMPX
   movf TEMP_ex, W    
   movwf ex
   movf TEMPX, W
   return
pop_w:                   ;pop_w no altera al registro STATUS.
   swapf TEMP_W, F       ;TEMP_W = swap(TEMP_W)    
   swapf TEMP_W, W       ;W=swap(TEMP_W) 
   return 
pop_STATUS:              ;pop_STATUS, Restaura STATUS y no afecta el
   movwf TEMPX           ;registro STATUS ni a W.
   swapf TEMP_STATUS, W  ;Ver push_STATUS para entender esta linea.
   movwf STATUS            
   swapf TEMPX, F
   swapf TEMPX, W
   return

;*********************************************************************            
;* push_*(): Ponen un registro en el stack.                          *   
;*********************************************************************            
push_ax:
   movwf TEMPX       ;TEMPX=W
   movf ax, W        ;W=ax
   movwf TEMP_ax     ;TEMP_ax=W=ax
   movf TEMPX, W     ;W=TEMPX
   return
push_bx:
   movwf TEMPX
   movf bx, W    
   movwf TEMP_bx
   movf TEMPX, W
   return
push_cx:
   movwf TEMPX
   movf cx, W    
   movwf TEMP_cx
   movf TEMPX, W
   return
push_dx:
   movwf TEMPX
   movf dx, W    
   movwf TEMP_dx
   movf TEMPX, W
   return
push_ex:
   movwf TEMPX
   movf ex, W    
   movwf TEMP_ex
   movf TEMPX, W
   return
push_w:                 ;push_w y push_STATUS no modifican el registro
   movwf TEMP_W         ;STATUS. 
   return
push_STATUS:
   movwf TEMPX          ;TEMPX=W
   swapf STATUS, W      ;W=swap(STATUS)
   movwf TEMP_STATUS    ;TEMP_STATUS=W
   swapf TEMPX, F       ;TEMPX=swap(TEMPX)
   swapf TEMPX, W       ;W=swap(swap(TEMPX))=TEMPX
   return

;*********************************************************************            
   END   ;Fin de codigo/Archivo. (EOF?)
;*********************************************************************            



